"""Parser for Kotlin documentation comments."""

import re
from typing import Dict, List, Optional, Any
from pathlib import Path

from .base import BaseParser, DocItem

class KotlinParser(BaseParser):
    """Parser for Kotlin documentation comments."""

    def __init__(self, root_path: Path = None):
        """Initialize the parser."""
        if root_path:
            super().__init__(root_path)
        
        self.doc_pattern = re.compile(
            r'/\*\*.*?\*/\s*([^\n]*)',
            re.DOTALL
        )
        
        self.tag_patterns = {
            'param': r'@param\s+(\w+)\s+([^\n@]*)',
            'return': r'@return\s+([^\n@]*)',
            'throws': r'@throws\s+(\w+)\s+([^\n@]*)',
            'exception': r'@exception\s+(\w+)\s+([^\n@]*)',
            'see': r'@see\s+([^\n@]*)',
            'author': r'@author\s+([^\n@]*)',
            'since': r'@since\s+([^\n@]*)',
            'sample': r'@sample\s+([^\n@]*)',
        }
        
        self.code_patterns = {
            'class': re.compile(r'(?:public|private|internal|protected)?\s*(?:data|sealed|open|abstract)?\s*class\s+(\w+)'),
            'function': re.compile(r'(?:public|private|internal|protected)?\s*fun\s+(\w+)'),
            'property': re.compile(r'(?:public|private|internal|protected|const)?\s*(?:val|var)\s+(\w+)'),
            'interface': re.compile(r'(?:public|private|internal|protected)?\s*interface\s+(\w+)'),
            'enum': re.compile(r'(?:public|private|internal|protected)?\s*enum\s+class\s+(\w+)'),
            'object': re.compile(r'(?:public|private|internal|protected)?\s*object\s+(\w+)')
        }

    def get_file_extensions(self) -> List[str]:
        """Get Kotlin file extensions."""
        return ['.kt', '.kts']

    def parse_file(self, file_path: Path) -> List[DocItem]:
        """Parse a Kotlin source file and extract documentation.

        Args:
            file_path: Path to the source file

        Returns:
            List of documentation items
        """
        if isinstance(file_path, Path):
            file_path_str = str(file_path)
        else:
            file_path_str = file_path
        
        doc_dict = self._parse_file_to_dict(file_path_str)
        
        doc_items = []
        
        for section_type, items in doc_dict.items():
            for item in items:
                if section_type == 'classes':
                    item_type = 'class'
                elif section_type == 'functions':
                    item_type = 'function'
                elif section_type == 'properties':
                    item_type = 'property'
                elif section_type == 'interfaces':
                    item_type = 'interface'
                elif section_type == 'enums':
                    item_type = 'enum'
                elif section_type == 'objects':
                    item_type = 'object'
                else:
                    item_type = 'unknown'
                
                doc_item = DocItem(
                    name=item.get('name', 'Unknown'),
                    type=item_type,
                    description=item.get('description', ''),
                    signature=item.get('signature', None),
                    params=item.get('params', {}),
                    returns=item.get('returns', None),
                    examples=item.get('examples', []),
                    source_file=file_path_str,
                    line_number=None
                )
                
                doc_items.append(doc_item)
        
        return doc_items

    def _parse_file_to_dict(self, file_path: str) -> Dict:
        """Parse a Kotlin source file and extract documentation as a dictionary.

        Args:
            file_path: Path to the source file

        Returns:
            Dict containing parsed documentation
        """
        with open(file_path, 'r', encoding='utf-8') as f:
            content = f.read()

        documentation = {
            'classes': [],
            'functions': [],
            'properties': [],
            'interfaces': [],
            'enums': [],
            'objects': []
        }

        matches = self.doc_pattern.finditer(content)

        for match in matches:
            doc_comment = match.group(0)
            code_line = match.group(1) if len(match.groups()) > 0 else ""
            
            doc_block = self._parse_doc_block(doc_comment)
            
            element_type, name = self._detect_code_element(code_line)
            
            if element_type and name:
                doc_block['name'] = name
                
                if element_type == 'class':
                    documentation['classes'].append(doc_block)
                elif element_type == 'function':
                    documentation['functions'].append(doc_block)
                elif element_type == 'property':
                    documentation['properties'].append(doc_block)
                elif element_type == 'interface':
                    documentation['interfaces'].append(doc_block)
                elif element_type == 'enum':
                    documentation['enums'].append(doc_block)
                elif element_type == 'object':
                    documentation['objects'].append(doc_block)

        return documentation
    
    def _parse_doc_block(self, doc_text: str) -> Dict[str, Any]:
        """Parse a KDoc comment block.
        
        Args:
            doc_text: The documentation text
            
        Returns:
            Dictionary with parsed documentation
        """
        doc_text = re.sub(r'/\*\*|\*/|^\s*\*', '', doc_text, flags=re.MULTILINE)
        doc_text = re.sub(r'^\s*', '', doc_text, flags=re.MULTILINE)
        
        description_match = re.search(r'^(.*?)(?=@|$)', doc_text, re.DOTALL)
        description = description_match.group(1).strip() if description_match else ""
        
        doc_block = {
            'description': description,
            'params': {},
            'returns': None,
            'throws': {},
            'examples': []
        }
        
        param_matches = re.finditer(self.tag_patterns['param'], doc_text)
        for match in param_matches:
            param_name = match.group(1)
            param_desc = match.group(2).strip()
            doc_block['params'][param_name] = param_desc
        
        return_match = re.search(self.tag_patterns['return'], doc_text)
        if return_match:
            doc_block['returns'] = return_match.group(1).strip()
        
        throws_matches = re.finditer(self.tag_patterns['throws'], doc_text)
        for match in throws_matches:
            exception_type = match.group(1)
            exception_desc = match.group(2).strip()
            doc_block['throws'][exception_type] = exception_desc
            
        exception_matches = re.finditer(self.tag_patterns['exception'], doc_text)
        for match in exception_matches:
            exception_type = match.group(1)
            exception_desc = match.group(2).strip()
            doc_block['throws'][exception_type] = exception_desc
        
        sample_matches = re.finditer(self.tag_patterns['sample'], doc_text)
        for match in sample_matches:
            doc_block['examples'].append(match.group(1).strip())
        
        return doc_block
    
    def _detect_code_element(self, code_line: str) -> tuple:
        """Detect the type of code element and its name.
        
        Args:
            code_line: The code line following the documentation
            
        Returns:
            Tuple of (element_type, name)
        """
        if not code_line:
            return None, None
            
        for element_type, pattern in self.code_patterns.items():
            match = pattern.search(code_line)
            if match:
                return element_type, match.group(1)
                
        return None, None 


"""Command-line interface for TheDoc."""

import click
from pathlib import Path

from .mkdocs_generator import MkDocsGenerator
from .config import load_config, save_config, DEFAULT_CONFIG

@click.group()
def main():
    """TheDoc - Documentation generation tool."""
    pass

@main.command()
def init():
    """Initialize TheDoc in the current project."""
    click.echo("Initializing TheDoc...")
    
    project_name = click.prompt("Project name", default="")
    
    config = DEFAULT_CONFIG.copy()
    config["project_name"] = project_name
    
    save_config(config)
    
    click.echo("TheDoc initialized successfully.")

@main.command()
def generate():
    """Generate documentation for the project."""
    click.echo("Generating documentation...")
    
    project_path = Path.cwd()
    
    generator = MkDocsGenerator(project_path)
    
    generator.generate()

@main.command()
def release_notes():
    """Generate release notes based on conventional commits."""
    click.echo("Generating release notes...")
    # TODO: Implement release notes generation

@main.command()
def build():
    """Build MkDocs documentation site."""
    click.echo("Building documentation site...")
    
    config = load_config()
    docs_dir = Path.cwd() / config["output_dir"]
    
    if not docs_dir.exists():
        click.echo(f"Documentation directory not found: {docs_dir}")
        click.echo("Run 'thedoc generate' first to generate documentation.")
        return
    
    import subprocess
    try:
        subprocess.run(["mkdocs", "build"], cwd=docs_dir, check=True)
        click.echo(f"Documentation site built successfully in {docs_dir}/site")
    except subprocess.CalledProcessError as e:
        click.echo(f"Error building documentation site: {e}")
    except FileNotFoundError:
        click.echo("MkDocs not found. Install it with 'pip install mkdocs mkdocs-material'.")

@main.command()
def serve():
    """Start the MkDocs development server."""
    click.echo("Starting documentation server...")
    
    config = load_config()
    docs_dir = Path.cwd() / config["output_dir"]
    
    if not docs_dir.exists():
        click.echo(f"Documentation directory not found: {docs_dir}")
        click.echo("Run 'thedoc generate' first to generate documentation.")
        return
    
    import subprocess
    try:
        subprocess.run(["mkdocs", "serve"], cwd=docs_dir, check=True)
    except subprocess.CalledProcessError as e:
        click.echo(f"Error serving documentation site: {e}")
    except FileNotFoundError:
        click.echo("MkDocs not found. Install it with 'pip install mkdocs mkdocs-material'.")

if __name__ == "__main__":
    main() 


"""Base parser for code analysis."""

from abc import ABC, abstractmethod
from dataclasses import dataclass
from pathlib import Path
from typing import Dict, List, Optional


@dataclass
class DocItem:
    """Represents a documentation item."""
    name: str
    type: str
    description: str
    signature: Optional[str] = None
    params: Dict[str, str] = None
    returns: Optional[str] = None
    examples: List[str] = None
    source_file: Optional[str] = None
    line_number: Optional[int] = None

    def __post_init__(self):
        """Initialize optional fields."""
        self.params = self.params or {}
        self.examples = self.examples or []


class BaseParser(ABC):
    """Base class for language-specific parsers."""

    def __init__(self, root_path: Path):
        """Initialize the parser.
        
        Args:
            root_path: The root path of the project to parse.
        """
        self.root_path = root_path

    @abstractmethod
    def parse_file(self, file_path: Path) -> List[DocItem]:
        """Parse a single file and extract documentation items.
        
        Args:
            file_path: Path to the file to parse.
            
        Returns:
            List of documentation items found in the file.
        """
        pass

    @abstractmethod
    def get_file_extensions(self) -> List[str]:
        """Get the file extensions this parser can handle.
        
        Returns:
            List of file extensions (e.g., ['.py', '.pyx']).
        """
        pass

    def is_supported_file(self, file_path: Path) -> bool:
        """Check if the given file is supported by this parser.
        
        Args:
            file_path: Path to the file to check.
            
        Returns:
            True if the file is supported, False otherwise.
        """
        return file_path.suffix.lower() in self.get_file_extensions()

    def parse_directory(self, directory: Path) -> List[DocItem]:
        """Parse all supported files in a directory recursively.
        
        Args:
            directory: Path to the directory to parse.
            
        Returns:
            List of documentation items found in all supported files.
        """
        items = []
        for file_path in directory.rglob("*"):
            if file_path.is_file() and self.is_supported_file(file_path):
                items.extend(self.parse_file(file_path))
        return items 


"""Test script for the .NET parser."""

from thedoc.parsers.dotnet_parser import DotNetParser
import json

def print_element_list(elements, indent=2):
    """Print a list of elements with their attributes."""
    for elem in elements:
        print(f"{' ' * indent}- Text: {elem['text']}")
        if elem['attributes']:
            print(f"{' ' * (indent + 2)}Attributes: {elem['attributes']}")

def main():
    """Run the parser on our sample file."""
    parser = DotNetParser()
    result = parser.parse_file("tests/dotnet/sample.cs")
    
    # Print the results in a readable format
    print("\nParsed Documentation:")
    print("=" * 50)
    
    # Print classes
    print("\nClasses:")
    for cls in result['classes']:
        print(f"\nClass: {cls['name']}")
        print(f"Summary: {cls['summary']}")
        print(f"Remarks: {cls['remarks']}")
        print(f"Example: {cls['example']}")
        if cls['see_also']:
            print("See Also:")
            print_element_list(cls['see_also'])
        if cls['type_params']:
            print("Type Parameters:")
            print_element_list(cls['type_params'])
        if cls['inheritance']:
            print(f"Inheritance: {cls['inheritance']}")
        if cls['includes']:
            print("Includes:")
            print_element_list(cls['includes'])
    
    # Print methods
    print("\nMethods:")
    for method in result['methods']:
        print(f"\nMethod: {method['name']}")
        print(f"Summary: {method['summary']}")
        print("Parameters:")
        print_element_list(method['parameters'])
        print(f"Returns: {method['returns']}")
        print("Exceptions:")
        print_element_list(method['exceptions'])
        print(f"Remarks: {method['remarks']}")
        print(f"Example: {method['example']}")
        if method['type_params']:
            print("Type Parameters:")
            print_element_list(method['type_params'])
        if method['see_also']:
            print("See Also:")
            print_element_list(method['see_also'])
        if method['inheritance']:
            print(f"Inheritance: {method['inheritance']}")
        if method['includes']:
            print("Includes:")
            print_element_list(method['includes'])
    
    # Print properties
    print("\nProperties:")
    for prop in result['properties']:
        print(f"\nProperty: {prop['name']}")
        print(f"Summary: {prop['summary']}")
        print(f"Value: {prop['value']}")
        print(f"Remarks: {prop['remarks']}")
        print(f"Example: {prop['example']}")
        if prop['see_also']:
            print("See Also:")
            print_element_list(prop['see_also'])
        if prop['inheritance']:
            print(f"Inheritance: {prop['inheritance']}")
        if prop['includes']:
            print("Includes:")
            print_element_list(prop['includes'])
    
    # Print enums
    print("\nEnums:")
    for enum in result['enums']:
        print(f"\nEnum: {enum['name']}")
        print(f"Summary: {enum['summary']}")
        print(f"Remarks: {enum['remarks']}")
        print("Values:")
        print_element_list(enum['values'])
        if enum['see_also']:
            print("See Also:")
            print_element_list(enum['see_also'])
        if enum['inheritance']:
            print(f"Inheritance: {enum['inheritance']}")
        if enum['includes']:
            print("Includes:")
            print_element_list(enum['includes'])
    
    # Print types
    print("\nTypes:")
    for type_info in result['types']:
        print(f"\nType: {type_info['name']}")
        print(f"Summary: {type_info['summary']}")
        print(f"Remarks: {type_info['remarks']}")
        if type_info['type_params']:
            print("Type Parameters:")
            print_element_list(type_info['type_params'])
        if type_info['see_also']:
            print("See Also:")
            print_element_list(type_info['see_also'])
        if type_info['inheritance']:
            print(f"Inheritance: {type_info['inheritance']}")
        if type_info['includes']:
            print("Includes:")
            print_element_list(type_info['includes'])

if __name__ == "__main__":
    main() 


"""Parser for .NET documentation comments."""

import re
import xml.etree.ElementTree as ET
from typing import Dict, List, Optional, Any, Set
from pathlib import Path

from .base import BaseParser, DocItem

class DotNetParser(BaseParser):
    """Parser for .NET documentation comments."""

    def __init__(self, root_path: Path = None):
        """Initialize the parser."""
        if root_path:
            super().__init__(root_path)
        
        self.doc_pattern = re.compile(r'///.*?(?=\n(?!\s*///))|<(?:class|interface|method|property|enum|type)[\s\S]*?</(?:class|interface|method|property|enum|type)>', re.DOTALL)

    def get_file_extensions(self) -> List[str]:
        """Get .NET file extensions."""
        return ['.cs', '.vb']

    def parse_file(self, file_path: Path) -> List[DocItem]:
        """Parse a .NET source file and extract documentation.

        Args:
            file_path: Path to the source file

        Returns:
            List of documentation items
        """
        if isinstance(file_path, Path):
            file_path_str = str(file_path)
        else:
            file_path_str = file_path
        
        doc_dict = self._parse_file_to_dict(file_path_str)
        
        doc_items = []
        
        for section_type, items in doc_dict.items():
            for item in items:
                if section_type == 'classes':
                    item_type = 'class'
                elif section_type == 'methods':
                    item_type = 'function'
                elif section_type == 'properties':
                    item_type = 'property'
                elif section_type == 'enums':
                    item_type = 'enum'
                elif section_type == 'interfaces':
                    item_type = 'interface'
                elif section_type == 'types':
                    item_type = 'type'
                else:
                    item_type = 'unknown'
                
                doc_item = DocItem(
                    name=item.get('name', 'Unknown'),
                    type=item_type,
                    description=item.get('description', ''),
                    signature=item.get('signature', None),
                    params=item.get('params', {}),
                    returns=item.get('returns', None),
                    examples=item.get('examples', []),
                    source_file=file_path_str,
                    line_number=None
                )
                
                doc_items.append(doc_item)
        
        return doc_items

    def _parse_file_to_dict(self, file_path: str) -> Dict:
        """Parse a .NET source file and extract documentation as a dictionary.

        Args:
            file_path: Path to the source file

        Returns:
            Dict containing parsed documentation
        """
        with open(file_path, 'r', encoding='utf-8') as f:
            content = f.read()

        content = self._convert_triple_slash_to_xml(content)

        doc_blocks = self.doc_pattern.findall(content)

        documentation = {
            'classes': [],
            'methods': [],
            'properties': [],
            'enums': [],
            'interfaces': [],
            'types': []
        }

        for block in doc_blocks:
            try:
                root = ET.fromstring(block)
                
                if root.tag == 'class':
                    documentation['classes'].append(self._parse_class(root))
                elif root.tag == 'method':
                    documentation['methods'].append(self._parse_method(root))
                elif root.tag == 'property':
                    documentation['properties'].append(self._parse_property(root))
                elif root.tag == 'enum':
                    documentation['enums'].append(self._parse_enum(root))
                elif root.tag == 'interface':
                    documentation['interfaces'].append(self._parse_interface(root))
                elif root.tag == 'type':
                    documentation['types'].append(self._parse_type(root))
            except ET.ParseError as e:
                print(f"Error parsing documentation block: {e}")
                print(f"Block content:\n{block}")
                continue

        return documentation
        
    def _convert_triple_slash_to_xml(self, content: str) -> str:
        """Convert triple slash comments to XML format.
        
        Args:
            content: The source file content
            
        Returns:
            Source with triple slash comments converted to XML
        """
        lines = content.split('\n')
        
        i = 0
        result = []
        while i < len(lines):
            line = lines[i]
            
            if line.strip().startswith('///'):
                comment_lines = []
                while i < len(lines) and lines[i].strip().startswith('///'):
                    comment_line = lines[i].strip()[3:].strip()
                    comment_lines.append(comment_line)
                    i += 1
                

                if i < len(lines):
                    next_line = lines[i]
                    
                    xml_tag = 'type'
                    if 'class ' in next_line:
                        xml_tag = 'class'
                    elif 'interface ' in next_line:
                        xml_tag = 'interface'
                    elif 'enum ' in next_line:
                        xml_tag = 'enum'
                    elif 'void ' in next_line or ' => ' in next_line or ' return ' in next_line:
                        xml_tag = 'method'
                    elif 'property ' in next_line or 'get;' in next_line or 'set;' in next_line:
                        xml_tag = 'property'
                    
                    name_match = re.search(r'\b([A-Za-z0-9_]+)\b(?=\s*[(:{\s]|$)', next_line)
                    name = name_match.group(1) if name_match else 'Unknown'
                    
                    xml = f"<{xml_tag} name=\"{name}\">\n"
                    
                    summary_lines = []
                    other_lines = []
                    
                    for cline in comment_lines:
                        if cline.startswith('<param') or cline.startswith('<returns') or cline.startswith('<exception'):
                            other_lines.append(cline)
                        else:
                            summary_lines.append(cline)
                    
                    if summary_lines:
                        xml += "<summary>\n"
                        xml += "\n".join(summary_lines)
                        xml += "\n</summary>\n"
                    
                    xml += "\n".join(other_lines)
                    
                    xml += f"\n</{xml_tag}>"
                    
                    result.append(xml)
                    result.append(next_line)
                else:
                    result.extend(comment_lines)
            else:
                result.append(line)
                i += 1
                
        return '\n'.join(result)
    
    def _parse_class(self, element: ET.Element) -> Dict[str, Any]:
        """Parse a class documentation element.
        
        Args:
            element: The XML element containing class documentation
            
        Returns:
            Dictionary with parsed class documentation
        """
        result = {
            'name': element.get('name', 'Unknown'),
            'description': '',
            'params': {},
            'examples': []
        }
        
        summary = element.find('summary')
        if summary is not None:
            result['description'] = ''.join(summary.itertext()).strip()
        
        examples = element.findall('example')
        for example in examples:
            result['examples'].append(''.join(example.itertext()).strip())
        
        return result
    
    def _parse_method(self, element: ET.Element) -> Dict[str, Any]:
        """Parse a method documentation element.
        
        Args:
            element: The XML element containing method documentation
            
        Returns:
            Dictionary with parsed method documentation
        """
        result = {
            'name': element.get('name', 'Unknown'),
            'description': '',
            'params': {},
            'returns': None,
            'examples': []
        }
        
        summary = element.find('summary')
        if summary is not None:
            result['description'] = ''.join(summary.itertext()).strip()
        
        params = element.findall('param')
        for param in params:
            name = param.get('name', 'unknown')
            description = ''.join(param.itertext()).strip()
            result['params'][name] = description
        
        returns = element.find('returns')
        if returns is not None:
            result['returns'] = ''.join(returns.itertext()).strip()
        
        examples = element.findall('example')
        for example in examples:
            result['examples'].append(''.join(example.itertext()).strip())
        
        return result
    
    def _parse_property(self, element: ET.Element) -> Dict[str, Any]:
        """Parse a property documentation element."""
        result = {
            'name': element.get('name', 'Unknown'),
            'description': '',
            'examples': []
        }
        
        summary = element.find('summary')
        if summary is not None:
            result['description'] = ''.join(summary.itertext()).strip()
        
        examples = element.findall('example')
        for example in examples:
            result['examples'].append(''.join(example.itertext()).strip())
        
        return result
    
    def _parse_enum(self, element: ET.Element) -> Dict[str, Any]:
        """Parse an enum documentation element."""
        return {
            'name': element.get('name', 'Unknown'),
            'description': self._get_summary_text(element),
            'values': {}
        }
    
    def _parse_interface(self, element: ET.Element) -> Dict[str, Any]:
        """Parse an interface documentation element."""
        return {
            'name': element.get('name', 'Unknown'),
            'description': self._get_summary_text(element)
        }
    
    def _parse_type(self, element: ET.Element) -> Dict[str, Any]:
        """Parse a type documentation element."""
        return {
            'name': element.get('name', 'Unknown'),
            'description': self._get_summary_text(element)
        }
    
    def _get_summary_text(self, element: ET.Element) -> str:
        """Get the summary text from an element."""
        summary = element.find('summary')
        if summary is not None:
            return ''.join(summary.itertext()).strip()
        return '' 


"""Tests for the Python parser."""

from pathlib import Path
import textwrap
from thedoc.parsers import PythonParser

def test_parse_function():
    """Test parsing a Python function with docstring."""
    code = textwrap.dedent('''
        def hello(name: str, age: int = None) -> str:
            """Say hello to someone.
            
            Args:
                name: The name of the person
                age: Optional age of the person
                
            Returns:
                A greeting message
                
            Example:
                >>> hello("Alice", 25)
                'Hello Alice, you are 25 years old!'
            """
            if age is not None:
                return f"Hello {name}, you are {age} years old!"
            return f"Hello {name}!"
    ''')
    
    # Create a temporary file
    tmp_file = Path('test_func.py')
    try:
        tmp_file.write_text(code)
        
        parser = PythonParser(Path('.'))
        items = parser.parse_file(tmp_file)
        
        assert len(items) == 1
        item = items[0]
        
        assert item.name == 'hello'
        assert item.type == 'function'
        assert 'Say hello to someone' in item.description
        assert item.signature == 'hello(name, age)'
        assert len(item.params) == 2
        assert 'name' in item.params
        assert 'age' in item.params
        assert item.returns is not None
        assert len(item.examples) == 1
    finally:
        tmp_file.unlink()

def test_parse_class():
    """Test parsing a Python class with methods."""
    code = textwrap.dedent('''
        class Person:
            """A class representing a person.
            
            This class stores basic information about a person.
            """
            
            def __init__(self, name: str):
                """Initialize a person.
                
                Args:
                    name: The person's name
                """
                self.name = name
                
            def greet(self) -> str:
                """Generate a greeting.
                
                Returns:
                    A greeting message
                """
                return f"Hello, I'm {self.name}!"
    ''')
    
    # Create a temporary file
    tmp_file = Path('test_class.py')
    try:
        tmp_file.write_text(code)
        
        parser = PythonParser(Path('.'))
        items = parser.parse_file(tmp_file)
        
        assert len(items) == 3  # Class + 2 methods
        
        # Check class
        class_item = next(item for item in items if item.type == 'class')
        assert class_item.name == 'Person'
        assert 'representing a person' in class_item.description
        
        # Check methods
        init_method = next(item for item in items if item.name == 'Person.__init__')
        assert init_method.type == 'method'
        assert len(init_method.params) == 1
        assert 'name' in init_method.params
        
        greet_method = next(item for item in items if item.name == 'Person.greet')
        assert greet_method.type == 'method'
        assert greet_method.returns is not None
    finally:
        tmp_file.unlink()

def test_parse_module():
    """Test parsing a Python module with docstring."""
    code = textwrap.dedent('''
        """A test module.
        
        This module contains test code.
        """
        
        def func():
            pass
    ''')
    
    # Create a temporary file
    tmp_file = Path('test_module.py')
    try:
        tmp_file.write_text(code)
        
        parser = PythonParser(Path('.'))
        items = parser.parse_file(tmp_file)
        
        assert len(items) == 1
        assert items[0].type == 'module'
        assert items[0].name == 'test_module'
        assert 'test module' in items[0].description.lower()
    finally:
        tmp_file.unlink() 


"""Tests for the MkDocs generator."""

import os
import shutil
import tempfile
from pathlib import Path
import pytest

from thedoc.mkdocs_generator import MkDocsGenerator
from thedoc.config import DEFAULT_CONFIG

class TestMkDocsGenerator:
    """Tests for the MkDocs generator."""
    
    @pytest.fixture
    def setup_test_project(self):
        """Setup a test project with sample files."""
        # Create a temporary directory
        temp_dir = tempfile.mkdtemp()
        try:
            # Create a test project structure
            project_path = Path(temp_dir)
            
            # Create a Python file with docstrings
            py_dir = project_path / "src" / "sample"
            py_dir.mkdir(parents=True)
            
            py_file = py_dir / "sample.py"
            py_file.write_text('''"""Sample module docstring."""

class SampleClass:
    """A sample class for testing.
    
    This class demonstrates how docstrings are parsed.
    """
    
    def sample_method(self, param1, param2):
        """Sample method with parameters.
        
        Args:
            param1: The first parameter.
            param2: The second parameter.
            
        Returns:
            A sample return value.
        """
        return param1 + param2
''')
            
            # Create a Swift file with documentation comments
            swift_dir = project_path / "Sources"
            swift_dir.mkdir(parents=True)
            
            swift_file = swift_dir / "Sample.swift"
            swift_file.write_text('''/// A sample Swift class.
/// This class demonstrates how Swift documentation is parsed.
public class SampleSwift {
    
    /// A sample property with documentation.
    var sampleProperty: String
    
    /// Creates a new instance of SampleSwift.
    /// - Parameters:
    ///   - value: The initial value for the property.
    public init(value: String) {
        self.sampleProperty = value
    }
    
    /// A sample method with parameters and return value.
    /// - Parameters:
    ///   - param1: The first parameter.
    ///   - param2: The second parameter.
    /// - Returns: The combined result of the parameters.
    public func sampleMethod(param1: String, param2: String) -> String {
        return param1 + param2
    }
}
''')
            
            yield project_path
        finally:
            # Clean up
            shutil.rmtree(temp_dir)
    
    def test_detect_project_languages(self, setup_test_project, monkeypatch):
        """Test language detection."""
        project_path = setup_test_project
        
        # Mock config loading
        def mock_load_config():
            return DEFAULT_CONFIG.copy()
        
        monkeypatch.setattr("thedoc.config.load_config", mock_load_config)
        
        # Create generator
        generator = MkDocsGenerator(project_path)
        
        # Detect languages
        detected = generator.detect_project_languages()
        
        # Check results
        assert "Python" in detected
        assert "Swift" in detected
    
    def test_generate_markdown(self, setup_test_project, monkeypatch):
        """Test markdown generation."""
        project_path = setup_test_project
        
        # Mock config loading
        def mock_load_config():
            config = DEFAULT_CONFIG.copy()
            config["project_name"] = "Test Project"
            return config
        
        monkeypatch.setattr("thedoc.config.load_config", mock_load_config)
        
        # Create generator
        generator = MkDocsGenerator(project_path)
        
        # Generate documentation
        generator.generate()
        
        # Check that docs directory was created
        docs_dir = project_path / "docs"
        assert docs_dir.exists()
        
        # Check that mkdocs.yml was created
        mkdocs_config = docs_dir / "mkdocs.yml"
        assert mkdocs_config.exists()
        
        # Check that markdown files were created
        docs_docs_dir = docs_dir / "docs"
        assert docs_docs_dir.exists()
        
        # Check index.md
        index = docs_docs_dir / "index.md"
        assert index.exists()
        assert "Test Project" in index.read_text()
        
        # Check that language directories were created
        python_dir = docs_docs_dir / "python"
        swift_dir = docs_docs_dir / "swift"
        
        assert python_dir.exists()
        assert swift_dir.exists() 


"""Python-specific code parser."""

import ast
from pathlib import Path
from typing import List, Optional, Dict, Any

from .base import BaseParser, DocItem


class PythonParser(BaseParser):
    """Parser for Python source code."""

    def get_file_extensions(self) -> List[str]:
        """Get Python file extensions."""
        return ['.py']

    def parse_file(self, file_path: Path) -> List[DocItem]:
        """Parse a Python file and extract documentation."""
        with open(file_path, 'r', encoding='utf-8') as f:
            content = f.read()

        try:
            tree = ast.parse(content)
            items = self._parse_module(tree, str(file_path))
            return [item for item in items if item is not None]
        except SyntaxError:
            print(f"Error parsing {file_path}: Invalid Python syntax")
            return []

    def _parse_module(self, tree: ast.Module, file_path: str) -> List[DocItem]:
        """Parse an AST module node."""
        items = []
        
        module_doc = ast.get_docstring(tree)
        if module_doc:
            items.append(DocItem(
                name=Path(file_path).stem,
                type='module',
                description=module_doc,
                source_file=file_path,
                line_number=1
            ))

        for node in tree.body:
            if isinstance(node, ast.ClassDef):
                items.extend(self._parse_class(node, file_path))
            elif isinstance(node, ast.FunctionDef):
                func_item = self._parse_function(node, file_path)
                if func_item:
                    items.append(func_item)

        return items

    def _parse_class(self, node: ast.ClassDef, file_path: str) -> List[DocItem]:
        """Parse a class definition."""
        items = []

        class_doc = ast.get_docstring(node)
        if class_doc:
            items.append(DocItem(
                name=node.name,
                type='class',
                description=class_doc,
                source_file=file_path,
                line_number=node.lineno
            ))

        for item in node.body:
            if isinstance(item, ast.FunctionDef):
                method = self._parse_function(item, file_path, is_method=True)
                if method:
                    method.name = f"{node.name}.{method.name}"
                    items.append(method)

        return items

    def _parse_function(self, node: ast.FunctionDef, file_path: str, is_method: bool = False) -> Optional[DocItem]:
        """Parse a function definition."""
        doc = ast.get_docstring(node)
        if not doc:
            return None

        args = []
        for arg in node.args.args:
            args.append(arg.arg)
        
        if node.args.vararg:
            args.append(f"*{node.args.vararg.arg}")
        if node.args.kwarg:
            args.append(f"**{node.args.kwarg.arg}")

        signature = f"{node.name}({', '.join(args)})"

        params: Dict[str, str] = {}
        returns: Optional[str] = None
        examples: List[str] = []
        description = []
        
        current_section = description
        lines = doc.split('\n')
        
        current_example = []
        in_example = False
        
        for line in lines:
            line = line.strip()
            if line.lower().startswith('args:') or line.lower().startswith('parameters:'):
                current_section = []
                in_example = False
            elif line.lower().startswith('returns:'):
                returns = line[8:].strip()
                in_example = False
            elif line.lower().startswith('example:') or line.lower().startswith('examples:'):
                in_example = True
                current_example = []
            elif line and ':' in line and not in_example and current_section != description:
                param, desc = line.split(':', 1)
                params[param.strip()] = desc.strip()
            elif line and in_example:
                current_example.append(line)
            elif not line and in_example and current_example:
                examples.append('\n'.join(current_example))
                current_example = []
            elif line and not in_example:
                current_section.append(line)
        
        if current_example:
            examples.append('\n'.join(current_example))

        return DocItem(
            name=node.name,
            type='method' if is_method else 'function',
            description='\n'.join(description).strip(),
            signature=signature,
            params=params,
            returns=returns,
            examples=examples,
            source_file=file_path,
            line_number=node.lineno
        ) 


"""MkDocs generator for TheDoc.

This module handles the generation of MkDocs documentation from extracted code documentation.
"""

import os
import yaml
from pathlib import Path
from typing import Dict, List, Any, Optional
import shutil

from .parsers import BaseParser, DocItem, PythonParser, SwiftParser, KotlinParser, DotNetParser
from .config import load_config

class MkDocsGenerator:
    """Generates MkDocs documentation from extracted code documentation."""
    
    def __init__(self, project_path: Path):
        """Initialize the MkDocs generator.
        
        Args:
            project_path: Path to the project root
        """
        self.project_path = project_path
        self.config = load_config()
        self.docs_dir = project_path / self.config["output_dir"]
        self.mkdocs_dir = self.docs_dir / "docs"
        self.mkdocs_config_file = self.docs_dir / "mkdocs.yml"
        
        self.parser_map = {
            '.py': PythonParser(project_path),
            '.swift': SwiftParser(project_path),
            '.kt': KotlinParser(project_path),
            '.kts': KotlinParser(project_path),
            '.cs': DotNetParser(project_path),
            '.vb': DotNetParser(project_path),
        }
        
        self.plural_map = {
            'class': 'Classes',
            'function': 'Functions',
            'method': 'Methods',
            'property': 'Properties',
            'module': 'Modules',
            'enum': 'Enumerations',
            'interface': 'Interfaces',
            'case': 'Cases',
            'type': 'Types',
            'object': 'Objects',
            'unknown': 'Miscellaneous'
        }
        
        self.detected_languages = set()
        
    def detect_project_languages(self) -> List[str]:
        """Detect programming languages used in the project.
        
        Returns:
            List of detected programming languages
        """
        language_extensions = {
            '.py': 'Python',
            '.swift': 'Swift',
            '.kt': 'Kotlin',
            '.kts': 'Kotlin',
            '.java': 'Java',
            '.cs': 'C#',
            '.vb': 'Visual Basic',
            '.go': 'Go',
            '.rs': 'Rust',
            '.js': 'JavaScript',
            '.ts': 'TypeScript'
        }
        
        detected = set()
        
        for file_path in self.project_path.rglob("*"):
            if file_path.is_file() and file_path.suffix in language_extensions:
                detected.add(language_extensions[file_path.suffix])
                self.detected_languages.add(file_path.suffix)
        
        return list(detected)
    
    def extract_documentation(self) -> Dict[str, List[DocItem]]:
        """Extract documentation from all supported files in the project.
        
        Returns:
            Dictionary containing all extracted documentation grouped by language
        """
        all_docs = {}
        
        for ext in self.detected_languages:
            if ext in self.parser_map:
                parser = self.parser_map[ext]
                lang_name = next((l for e, l in {
                    '.py': 'Python', 
                    '.swift': 'Swift', 
                    '.kt': 'Kotlin',
                    '.kts': 'Kotlin',
                    '.cs': 'C#',
                    '.vb': 'Visual Basic'
                }.items() if e == ext), ext)
                
                print(f"Extracting documentation for {lang_name} files...")
                
                files = [f for f in self.project_path.rglob(f"*{ext}") 
                         if not any(pattern in str(f) for pattern in self.config["exclude_patterns"])]
                
                file_docs = {}
                for file_path in files:
                    try:
                        relative_path = file_path.relative_to(self.project_path)
                        doc_items = parser.parse_file(file_path)
                        if doc_items:
                            file_docs[str(relative_path)] = doc_items
                    except Exception as e:
                        print(f"Error parsing {file_path}: {e}")
                
                if file_docs:
                    all_docs[lang_name] = file_docs
        
        return all_docs
    
    def generate_markdown(self, documentation: Dict[str, Dict[str, List[DocItem]]]) -> None:
        """Generate Markdown files from extracted documentation.
        
        Args:
            documentation: Dictionary containing all extracted documentation
        """
        self.mkdocs_dir.mkdir(parents=True, exist_ok=True)
        
        self._create_index_file()
        
        for language, files in documentation.items():
            lang_dir = self.mkdocs_dir / language.lower().replace('#', 'sharp').replace('+', 'plus')
            lang_dir.mkdir(exist_ok=True)
            
            self._create_language_index(language, lang_dir, files)
            
            for file_path, doc_items in files.items():
                self._create_file_documentation(language, file_path, doc_items, lang_dir)
    
    def _create_index_file(self) -> None:
        """Create the main index.md file for the documentation."""
        index_content = f"""# {self.config['project_name'] or 'Project'} Documentation

This documentation was automatically generated by TheDoc.

## Project Overview

This project contains code in the following languages:

{chr(10).join(['* ' + lang for lang in self.detect_project_languages()])}

## Documentation Structure

The documentation is organized by programming language. Each language section contains
documentation for all files with documentation comments.

"""
        with open(self.mkdocs_dir / "index.md", "w") as f:
            f.write(index_content)
    
    def _create_language_index(self, language: str, lang_dir: Path, files: Dict[str, List[DocItem]]) -> None:
        """Create the index.md file for a specific language.
        
        Args:
            language: The programming language
            lang_dir: The directory for the language documentation
            files: Dictionary of files with their documentation items
        """
        index_content = f"""# {language} Documentation

This section contains documentation for {language} code in the project.

## Files

"""
        for file_path in sorted(files.keys()):
            safe_filename = file_path.replace("/", "_").replace("\\", "_")
            index_content += f"* [{file_path}]({safe_filename}.md)\n"
        
        with open(lang_dir / "index.md", "w") as f:
            f.write(index_content)
    
    def _create_file_documentation(self, language: str, file_path: str, 
                                 doc_items: List[DocItem], lang_dir: Path) -> None:
        """Create Markdown documentation for a specific file.
        
        Args:
            language: The programming language
            file_path: The path to the source file
            doc_items: List of documentation items for the file
            lang_dir: The directory for the language documentation
        """
        safe_filename = file_path.replace("/", "_").replace("\\", "_")
        md_file = lang_dir / f"{safe_filename}.md"
        
        with open(md_file, "w") as f:
            f.write(f"# {os.path.basename(file_path)}\n\n")
            f.write(f"**Path:** `{file_path}`\n\n")
            
            items_by_type = {}
            for item in doc_items:
                if item.type not in items_by_type:
                    items_by_type[item.type] = []
                items_by_type[item.type].append(item)
            
            for item_type, items in sorted(items_by_type.items()):
                if items:
                    section_title = self.plural_map.get(item_type, f"{item_type.capitalize()}s")
                    f.write(f"## {section_title}\n\n")
                    
                    for item in items:
                        f.write(f"### {item.name}\n\n")
                        
                        if item.description:
                            f.write(f"{item.description}\n\n")
                        
                        if item.signature:
                            f.write("```%s\n%s\n```\n\n" % (language.lower().replace('#', 'csharp').replace('+', 'cpp'), item.signature))
                        
                        if item.params:
                            f.write("**Parameters:**\n\n")
                            for param_name, param_desc in item.params.items():
                                f.write(f"* `{param_name}`: {param_desc}\n")
                            f.write("\n")
                        
                        if item.returns:
                            f.write(f"**Returns:** {item.returns}\n\n")
                        
                        if item.examples:
                            f.write("**Examples:**\n\n")
                            for example in item.examples:
                                f.write("```%s\n%s\n```\n\n" % (language.lower().replace('#', 'csharp').replace('+', 'cpp'), example))
                        
                        f.write("---\n\n")
    
    def create_mkdocs_config(self) -> None:
        """Create the MkDocs configuration file."""
        config = {
            "site_name": self.config['project_name'] or "Project Documentation",
            "theme": {
                "name": "material",
                "palette": {
                    "primary": "indigo",
                    "accent": "indigo"
                },
                "features": [
                    "navigation.tabs",
                    "navigation.sections",
                    "toc.integrate"
                ]
            },
            "markdown_extensions": [
                "pymdownx.highlight",
                "pymdownx.superfences",
                "admonition",
                "toc"
            ],
            "nav": [
                {"Home": "index.md"}
            ]
        }
        
        for language in self.detect_project_languages():
            lang_path = language.lower().replace('#', 'sharp').replace('+', 'plus')
            config["nav"].append({language: f"{lang_path}/index.md"})
        
        with open(self.mkdocs_config_file, "w") as f:
            yaml.dump(config, f, default_flow_style=False)
    
    def generate(self) -> None:
        """Generate the MkDocs documentation."""
        print("Detecting programming languages in the project...")
        detected = self.detect_project_languages()
        print(f"Detected languages: {', '.join(detected)}")
        
        print("Extracting documentation...")
        documentation = self.extract_documentation()
        
        if self.mkdocs_dir.exists():
            shutil.rmtree(self.mkdocs_dir)
        
        print("Generating Markdown files...")
        self.generate_markdown(documentation)
        
        print("Creating MkDocs configuration...")
        self.create_mkdocs_config()
        
        print(f"Documentation generated successfully in {self.docs_dir}")
        print("Run 'cd docs && mkdocs serve' to view the documentation") 


"""Tests for the .NET parser."""

import os
from pathlib import Path
import pytest
from thedoc.parsers.dotnet_parser import DotNetParser

@pytest.fixture
def parser():
    """Create a parser instance."""
    return DotNetParser()

@pytest.fixture
def sample_cs_file(tmp_path):
    """Create a sample C# file for testing."""
    file_path = tmp_path / "calculator.cs"
    with open(file_path, "w") as f:
        f.write("""
/// <class name="Calculator">
/// <summary>
/// Performs basic arithmetic operations.
/// </summary>
/// <remarks>
/// This class is thread-safe.
/// </remarks>
/// <example>
/// var calc = new Calculator();
/// var result = calc.Add(2, 3);
/// </example>
/// </class>
public class Calculator
{
    /// <method name="Add">
    /// <summary>
    /// Adds two numbers.
    /// </summary>
    /// <param name="a">First number</param>
    /// <param name="b">Second number</param>
    /// <returns>The sum of the two numbers.</returns>
    /// </method>
    public int Add(int a, int b)
    {
        return a + b;
    }
}
""")
    return file_path

def test_parse_file(parser, sample_cs_file):
    """Test parsing a C# file with documentation comments."""
    result = parser.parse_file(str(sample_cs_file))

    # Check classes
    assert len(result['classes']) == 1
    calculator_class = result['classes'][0]
    assert calculator_class['name'] == 'Calculator'
    assert 'Performs basic arithmetic operations' in calculator_class['summary']
    assert 'This class is thread-safe' in calculator_class['remarks']
    assert 'var calc = new Calculator()' in calculator_class['example']

    # Check methods
    assert len(result['methods']) == 1
    add_method = result['methods'][0]
    assert add_method['name'] == 'Add'
    assert 'Adds two numbers' in add_method['summary']
    assert len(add_method['parameters']) == 2
    assert add_method['parameters'][0]['attributes']['name'] == 'a'
    assert 'First number' in add_method['parameters'][0]['text']
    assert 'sum of the two numbers' in add_method['returns']

def test_parse_file_with_invalid_xml(parser, tmp_path):
    """Test parsing a file with invalid XML documentation."""
    file_path = tmp_path / "invalid.cs"
    with open(file_path, "w") as f:
        f.write("""
/// <class name="Invalid">
/// <summary>
/// This has invalid XML.
/// <unclosed>
/// </summary>
/// </class>
public class Invalid {}
""")
    
    result = parser.parse_file(str(file_path))
    assert result['classes'] == []  # Should gracefully handle invalid XML

def test_parse_file_with_no_documentation(parser, tmp_path):
    """Test parsing a file with no documentation."""
    file_path = tmp_path / "nodocs.cs"
    with open(file_path, "w") as f:
        f.write("""
public class NoDocumentation
{
    public void UndocumentedMethod() {}
}
""")
    
    result = parser.parse_file(str(file_path))
    assert result['classes'] == []
    assert result['methods'] == []

def test_parse_comprehensive_sample(parser):
    """Test parsing the comprehensive sample.cs file in the tests/dotnet directory."""
    sample_path = os.path.join("tests", "dotnet", "sample.cs")
    result = parser.parse_file(sample_path)
    
    # Check classes
    assert len(result['classes']) == 1
    math_class = result['classes'][0]
    assert math_class['name'] == 'MathOperations'
    assert 'mathematical operations' in math_class['summary']
    assert 'thread-safe' in math_class['remarks']
    
    # Check methods
    assert len(result['methods']) == 1
    multiply_method = result['methods'][0]
    assert multiply_method['name'] == 'Multiply'
    assert 'Multiplies two numbers' in multiply_method['summary']
    assert len(multiply_method['parameters']) == 2
    assert 'first number' in multiply_method['parameters'][0]['text'].lower()
    assert 'product of the two numbers' in multiply_method['returns']
    assert len(multiply_method['exceptions']) == 1
    assert 'ArgumentException' in multiply_method['exceptions'][0]['attributes']['cref']
    
    # Check properties
    assert len(result['properties']) == 1
    last_result_prop = result['properties'][0]
    assert last_result_prop['name'] == 'LastResult'
    assert 'Gets the result' in last_result_prop['summary']
    assert 'last calculated result' in last_result_prop['value']
    
    # Check enums
    assert len(result['enums']) == 1
    op_type_enum = result['enums'][0]
    assert op_type_enum['name'] == 'OperationType'
    assert 'types of operations' in op_type_enum['summary']
    
    # Check types/generics
    assert len(result['types']) == 1
    type_param = result['types'][0]
    assert type_param['name'] == 'T'
    assert 'generic type parameter' in type_param['summary'].lower()
    assert len(type_param['type_params']) == 1
    assert type_param['type_params'][0]['attributes']['name'] == 'T' 


"""Test script for the Swift parser."""

from thedoc.parsers.swift_parser import SwiftParser

def print_element_list(elements, indent=2):
    """Print a list of elements with their attributes."""
    for elem in elements:
        if 'name' in elem:
            print(f"{' ' * indent}- {elem['name']}: {elem['description']}")
        else:
            print(f"{' ' * indent}- {elem['text']}")

def main():
    """Run the parser on our sample file."""
    parser = SwiftParser()
    result = parser.parse_file("tests/swift/sample.swift")
    
    # Print the results in a readable format
    print("\nParsed Documentation:")
    print("=" * 50)
    
    # Print classes
    print("\nClasses:")
    for cls in result['classes']:
        print(f"\nClass: {cls['name']}")
        print(f"Description: {cls['description']}")
        if cls.get('examples'):
            print("Examples:")
            for example in cls['examples']:
                print(f"{' ' * 2}- ```swift\n{example}\n  ```")
        if cls['notes']:
            print("Notes:")
            print_element_list(cls['notes'])
        if cls['warnings']:
            print("Warnings:")
            print_element_list(cls['warnings'])
        if cls['see_also']:
            print("See Also:")
            print_element_list(cls['see_also'])
    
    # Print functions
    print("\nFunctions:")
    for func in result['functions']:
        print(f"\nFunction: {func['name']}")
        print(f"Description: {func['description']}")
        if func['parameters']:
            print("Parameters:")
            print_element_list(func['parameters'])
        if func['returns']:
            print("Returns:")
            print_element_list(func['returns'])
        if func['throws']:
            print("Throws:")
            print_element_list(func['throws'])
        if func.get('examples'):
            print("Examples:")
            for example in func['examples']:
                print(f"{' ' * 2}- ```swift\n{example}\n  ```")
        if func['preconditions']:
            print("Preconditions:")
            print_element_list(func['preconditions'])
        if func['postconditions']:
            print("Postconditions:")
            print_element_list(func['postconditions'])
    
    # Print properties
    print("\nProperties:")
    for prop in result['properties']:
        print(f"\nProperty: {prop['name']}")
        print(f"Description: {prop['description']}")
        if prop['important']:
            print("Important:")
            print_element_list(prop['important'])
    
    # Print enums
    print("\nEnums:")
    for enum in result['enums']:
        print(f"\nEnum: {enum['name']}")
        print(f"Description: {enum['description']}")
    
    # Print enum cases
    print("\nEnum Cases:")
    for case in result['cases']:
        print(f"\nCase: {case['name']}")
        print(f"Description: {case['description']}")
    
    # Print types
    print("\nTypes:")
    for type_info in result['types']:
        print(f"\nType: {type_info['name']}")
        print(f"Description: {type_info['description']}")
        if type_info.get('examples'):
            print("Examples:")
            for example in type_info['examples']:
                print(f"{' ' * 2}- ```swift\n{example}\n  ```")
        if type_info['notes']:
            print("Notes:")
            print_element_list(type_info['notes'])

if __name__ == "__main__":
    main() 


"""Tests for the CLI module."""

from click.testing import CliRunner
from thedoc.cli import main

def test_main_command():
    """Test that the main command runs without error."""
    runner = CliRunner()
    result = runner.invoke(main)
    assert result.exit_code == 0

def test_init_command():
    """Test the init command."""
    runner = CliRunner()
    result = runner.invoke(main, ["init"])
    assert result.exit_code == 0
    assert "Initializing TheDoc" in result.output

def test_generate_command():
    """Test the generate command."""
    runner = CliRunner()
    result = runner.invoke(main, ["generate"])
    assert result.exit_code == 0
    assert "Generating documentation" in result.output 


"""Configuration management for TheDoc."""

import os
import yaml
from pathlib import Path
from typing import Dict, Any

DEFAULT_CONFIG = {
    "project_name": "",
    "output_dir": "docs",
    "docs_dir": "",
    "exclude_patterns": [
        "*.pyc",
        "__pycache__",
        ".git",
        "venv",
        "node_modules",
    ],
    "supported_languages": [
        "python",
        "javascript",
        "typescript",
        "java",
        "csharp",
        "go",
        "rust",
    ],
}

def get_config_path() -> Path:
    """Get the path to the configuration file."""
    return Path.cwd() / "thedoc.yaml"

def load_config() -> Dict[str, Any]:
    """Load configuration from file or return defaults."""
    config_path = get_config_path()
    if not config_path.exists():
        return DEFAULT_CONFIG.copy()
    
    try:
        with open(config_path, 'r') as f:
            config = yaml.safe_load(f)
            
        merged_config = DEFAULT_CONFIG.copy()
        merged_config.update(config)
        return merged_config
    except Exception as e:
        print(f"Error loading config: {e}")
        return DEFAULT_CONFIG.copy()

def save_config(config: Dict[str, Any]) -> None:
    """Save configuration to file."""
    config_path = get_config_path()
    try:
        with open(config_path, 'w') as f:
            yaml.dump(config, f, default_flow_style=False)
    except Exception as e:
        print(f"Error saving config: {e}") 


"""Parser package for TheDoc."""

from .base import BaseParser, DocItem
from .python import PythonParser
from .swift_parser import SwiftParser
from .kotlin_parser import KotlinParser
from .dotnet_parser import DotNetParser

__all__ = [
    'BaseParser',
    'DocItem',
    'PythonParser',
    'SwiftParser',
    'KotlinParser',
    'DotNetParser',
] 


"""Parser for Swift documentation comments."""

import re
from typing import Dict, List, Optional, Any, Tuple
from pathlib import Path

from .base import BaseParser, DocItem

class SwiftParser(BaseParser):
    """Parser for Swift documentation comments."""

    def __init__(self, root_path: Path = None):
        """Initialize the parser."""
        if root_path:
            super().__init__(root_path)
        
        self.doc_pattern = re.compile(
            r'///.*?(?=\n(?!///)|$)|/\*\*.*?\*/',
            re.DOTALL | re.MULTILINE
        )
        
        self.code_line_pattern = re.compile(
            r'\n([^\n]+)',
            re.DOTALL
        )
        
        self.tag_patterns = {
            'Parameters': r'-\s*Parameters?\s*:([^-]*)',
            'Returns': r'-\s*Returns?\s*:([^-]*)',
            'Throws': r'-\s*Throws\s*:([^-]*)',
            'Note': r'-\s*Note\s*:([^-]*)',
            'Warning': r'-\s*Warning\s*:([^-]*)',
            'Important': r'-\s*Important\s*:([^-]*)',
            'SeeAlso': r'-\s*SeeAlso\s*:([^-]*)',
            'Precondition': r'-\s*Precondition\s*:([^-]*)',
            'Postcondition': r'-\s*Postcondition\s*:([^-]*)',
            'Case': r'-\s*Case\s*:([^-]*)',
        }
        
        self.parameter_pattern = re.compile(
            r'-\s*(\w+)\s*:([^-]*)',
            re.DOTALL
        )
        
        self.code_patterns = {
            'class': re.compile(r'(?:public|private|internal|fileprivate\s+)?class\s+(\w+)(?!\s*<)'),
            'function': re.compile(r'(?:(?:public|private|internal|fileprivate)\s+)?func\s+([a-zA-Z_][a-zA-Z0-9_]*)'),
            'property': re.compile(r'(?:private\(set\)\s+)?(?:var|let)\s+(\w+)'),
            'enum': re.compile(r'(?:public|private|internal|fileprivate\s+)?enum\s+(\w+)'),
            'case': re.compile(r'case\s+(\w+)(?:\s*=\s*[^,]+)?'),
            'type': re.compile(r'(?:public|private|internal|fileprivate\s+)?class\s+(\w+)(?:\s*<[^>]*>)?')
        }
        
        self.plural_map = {
            'class': 'classes',
            'function': 'functions',
            'property': 'properties',
            'enum': 'enums',
            'case': 'cases',
            'type': 'types'
        }

    def get_file_extensions(self) -> List[str]:
        """Get Swift file extensions."""
        return ['.swift']

    def parse_file(self, file_path: Path) -> List[DocItem]:
        """Parse a Swift source file and extract documentation.

        Args:
            file_path: Path to the source file

        Returns:
            List of documentation items
        """
        if isinstance(file_path, Path):
            file_path_str = str(file_path)
        else:
            file_path_str = file_path
            
        doc_dict = self._parse_file_to_dict(file_path_str)
        
        doc_items = []
        
        for section_type, items in doc_dict.items():
            for item in items:
                if section_type == 'classes':
                    item_type = 'class'
                elif section_type == 'functions':
                    item_type = 'function'
                elif section_type == 'properties':
                    item_type = 'property'
                elif section_type == 'enums':
                    item_type = 'enum'
                elif section_type == 'cases':
                    item_type = 'case'
                else:
                    item_type = 'unknown'
                
                doc_item = DocItem(
                    name=item.get('name', 'Unknown'),
                    type=item_type,
                    description=item.get('description', ''),
                    signature=item.get('signature', None),
                    params=item.get('params', {}),
                    returns=item.get('returns', None),
                    examples=item.get('examples', []),
                    source_file=file_path_str,
                    line_number=None
                )
                
                doc_items.append(doc_item)
        
        return doc_items
        
    def _parse_file_to_dict(self, file_path: str) -> Dict:
        """Parse a Swift source file and extract documentation as a dictionary.

        Args:
            file_path: Path to the source file

        Returns:
            Dict containing parsed documentation
        """
        with open(file_path, 'r', encoding='utf-8') as f:
            content = f.read()

        documentation = {
            'classes': [],
            'functions': [],
            'properties': [],
            'enums': [],
            'cases': [],
            'types': []
        }

        lines = content.split('\n')
        line_index = 0
        
        while line_index < len(lines):
            line = lines[line_index]
            
            if line.strip().startswith('///'):
                doc_lines = [line]
                line_index += 1
                
                while line_index < len(lines) and lines[line_index].strip().startswith('///'):
                    doc_lines.append(lines[line_index])
                    line_index += 1
                
                while line_index < len(lines) and not lines[line_index].strip():
                    line_index += 1
                
                if line_index >= len(lines):
                    break
                
                code_line = lines[line_index]
                
                if not code_line.strip().startswith('///') and not code_line.strip().startswith('/**'):
                    doc_text = '\n'.join(doc_lines)
                    self._process_doc_block(doc_text, code_line, documentation)
                
                line_index += 1
            
            elif line.strip().startswith('/**'):
                doc_text = line
                multiline_ended = False
                
                line_index += 1
                while line_index < len(lines) and not multiline_ended:
                    doc_text += '\n' + lines[line_index]
                    if '*/' in lines[line_index]:
                        multiline_ended = True
                    line_index += 1
                
                while line_index < len(lines) and not lines[line_index].strip():
                    line_index += 1
                
                if line_index >= len(lines):
                    break
                
                code_line = lines[line_index]
                
                self._process_doc_block(doc_text, code_line, documentation)
                
                line_index += 1
            else:
                line_index += 1

        return documentation

    def _process_doc_block(self, doc_text, code_line, documentation):
        """Process a documentation block and its associated code line.
        
        Args:
            doc_text: The documentation text
            code_line: The associated code line
            documentation: The documentation dictionary to update
        """
        doc_block = self._parse_doc_block(doc_text)
        
        element_type, name = self._detect_code_element(code_line)
        
        if element_type and name:
            doc_block['name'] = name
            plural_type = self.plural_map[element_type]
            

            if element_type == 'case':
                clean_text = self._clean_description(doc_text)
                lines = clean_text.split('\n')
                for line in lines:
                    if line.strip():
                        doc_block['description'] = line.strip()
                        break
            
            if element_type in ['property', 'function', 'enum', 'type', 'class']:
                clean_text = self._clean_description(doc_text)
                
                sections = re.split(r'\n##\s+', clean_text)
                
                if sections:
                    main_section = sections[0].strip()
                    
                    parts = re.split(r'\n(?=(?:-\s+(?:' + '|'.join(self.tag_patterns.keys()) + r')\s*:))', main_section)
                    if parts:
                        doc_block['description'] = parts[0].strip()
                
                for section in sections[1:]:
                    if not section.strip():
                        continue
                    
                    section_parts = section.split('\n', 1)
                    if len(section_parts) < 2:
                        continue
                    
                    title, content = section_parts
                    title = title.lower()
                    
                    if title == 'example':
                        code_blocks = re.findall(r'```swift\n(.*?)```', content, re.DOTALL)
                        if code_blocks:
                            doc_block['examples'] = [block.strip() for block in code_blocks]
            
            documentation[plural_type].append(doc_block)

    def _parse_doc_block(self, doc_text: str) -> Dict:
        """Parse a documentation block.

        Args:
            doc_text: The documentation text to parse

        Returns:
            Dict containing parsed documentation
        """
        doc_text = self._clean_description(doc_text)
        
        doc_block = {
            'description': '',
            'parameters': [],
            'returns': [],
            'throws': [],
            'notes': [],
            'warnings': [],
            'important': [],
            'see_also': [],
            'preconditions': [],
            'postconditions': [],
            'cases': []
        }
        
        sections = re.split(r'\n##\s+', doc_text)
        
        main_section = sections[0].strip()
        
        parts = re.split(r'\n(?=(?:-\s+(?:' + '|'.join(self.tag_patterns.keys()) + r')\s*:))', main_section)
        if parts:
            description = parts[0].strip()
            doc_block['description'] = description
            
            if len(parts) > 1:
                tags_text = '\n'.join(parts[1:])
                self._parse_tags(tags_text, doc_block)
        
        for section in sections[1:]:
            if not section.strip():
                continue
            
            section_parts = section.split('\n', 1)
            if len(section_parts) < 2:
                continue
                
            title, content = section_parts
            title = title.lower()
            
            if title == 'example':
                code_blocks = re.findall(r'```swift\n(.*?)```', content, re.DOTALL)
                if code_blocks:
                    doc_block['examples'] = [block.strip() for block in code_blocks]
            elif title == 'parameters':
                param_matches = self.parameter_pattern.finditer(content)
                for match in param_matches:
                    doc_block['parameters'].append({
                        'name': match.group(1),
                        'description': match.group(2).strip()
                    })
            elif title == 'cases':
                case_lines = content.strip().split('\n')
                for line in case_lines:
                    case_match = re.match(r'-\s*`(\w+)`\s*:\s*(.+)', line.strip())
                    if case_match:
                        doc_block['cases'].append({
                            'name': case_match.group(1),
                            'description': case_match.group(2).strip()
                        })
        
        return doc_block

    def _parse_tags(self, tags_text: str, doc_block: Dict) -> None:
        """Parse documentation tags and add them to the doc block.

        Args:
            tags_text: The text containing the tags
            doc_block: The documentation block to update
        """
        for tag, pattern in self.tag_patterns.items():
            matches = re.finditer(pattern, tags_text, re.DOTALL)
            for match in matches:
                content = match.group(1).strip()
                if tag == 'Parameters':
                    param_matches = self.parameter_pattern.finditer(content)
                    for param_match in param_matches:
                        doc_block['parameters'].append({
                            'name': param_match.group(1),
                            'description': param_match.group(2).strip()
                        })
                elif tag == 'Case':
                    case_match = re.match(r'(\w+):\s*(.+)', content)
                    if case_match:
                        doc_block['cases'].append({
                            'name': case_match.group(1),
                            'description': case_match.group(2).strip()
                        })
                else:
                    tag_list = tag.lower() + 's'
                    if tag_list in doc_block:
                        doc_block[tag_list].append({'text': content})

    def _detect_code_element(self, code_line: str) -> Tuple[Optional[str], Optional[str]]:
        """Detect the type of Swift code element and its name.

        Args:
            code_line: The line of code following the doc comment

        Returns:
            Tuple of (element_type, element_name) or (None, None) if not detected
        """
        generic_class_match = re.search(r'(?:public\s+|private\s+|internal\s+|fileprivate\s+)?class\s+(\w+)\s*<', code_line)
        if generic_class_match:
            return 'type', generic_class_match.group(1)
        
        for element_type, pattern in self.code_patterns.items():
            match = pattern.search(code_line)
            if match:
                return element_type, match.group(1)
        
        return None, None

    def _clean_description(self, text: str) -> str:
        """Clean up the description text by removing comment markers and extra whitespace.

        Args:
            text: The description text to clean

        Returns:
            Cleaned description text
        """
        if text.startswith('/**'):
            text = re.sub(r'^\s*/\*\*\s*|\s*\*/\s*$', '', text)
            lines = text.split('\n')
            cleaned_lines = []
            for line in lines:
                line = re.sub(r'^\s*\*\s*', '', line)
                cleaned_lines.append(line.rstrip())
        else:
            lines = text.split('\n')
            cleaned_lines = []
            for line in lines:
                line = re.sub(r'^\s*///\s*', '', line)
                if not cleaned_lines and not line.strip():
                    continue
                cleaned_lines.append(line.rstrip())
        
        return '\n'.join(cleaned_lines).strip() 


"""Test script for the Kotlin parser."""

from thedoc.parsers.kotlin_parser import KotlinParser

def print_element_list(elements, indent=2):
    """Print a list of elements with their attributes."""
    for elem in elements:
        if 'name' in elem:
            print(f"{' ' * indent}- {elem['name']}: {elem['description']}")
        else:
            print(f"{' ' * indent}- {elem['text']}")

def main():
    """Run the parser on our sample file."""
    parser = KotlinParser()
    result = parser.parse_file("tests/kotlin/sample.kt")
    
    # Print the results in a readable format
    print("\nParsed Documentation:")
    print("=" * 50)
    
    # Print classes
    print("\nClasses:")
    for cls in result['classes']:
        print(f"\nClass: {cls['name']}")
        print(f"Description: {cls['description']}")
        if cls['properties']:
            print("Properties:")
            print_element_list(cls['properties'])
        if cls['see_also']:
            print("See Also:")
            print_element_list(cls['see_also'])
        if cls['samples']:
            print("Samples:")
            print_element_list(cls['samples'])
    
    # Print functions
    print("\nFunctions:")
    for func in result['functions']:
        print(f"\nFunction: {func['name']}")
        print(f"Description: {func['description']}")
        if func['parameters']:
            print("Parameters:")
            print_element_list(func['parameters'])
        if func['returns']:
            print("Returns:")
            print_element_list(func['returns'])
        if func['throws']:
            print("Throws:")
            print_element_list(func['throws'])
        if func['samples']:
            print("Samples:")
            print_element_list(func['samples'])
    
    # Print properties
    print("\nProperties:")
    for prop in result['properties']:
        print(f"\nProperty: {prop['name']}")
        print(f"Description: {prop['description']}")
        if prop['see_also']:
            print("See Also:")
            print_element_list(prop['see_also'])
    
    # Print enums
    print("\nEnums:")
    for enum in result['enums']:
        print(f"\nEnum: {enum['name']}")
        print(f"Description: {enum['description']}")
        if enum['properties']:
            print("Values:")
            print_element_list(enum['properties'])
    
    # Print types
    print("\nTypes:")
    for type_info in result['types']:
        print(f"\nType: {type_info['name']}")
        print(f"Description: {type_info['description']}")
        if type_info['parameters']:
            print("Type Parameters:")
            print_element_list(type_info['parameters'])
        if type_info['see_also']:
            print("See Also:")
            print_element_list(type_info['see_also'])

if __name__ == "__main__":
    main() 